package se.chalmers.tdv.test;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import se.chalmers.tdv.main.Queue;

public class QueueTest {
    
	public Queue q0;
	public Queue q2;
	
	@Before
	public void setUp() throws Exception {
		q0 = new Queue(0);
		q2 = new Queue(2);
	}
	
	/* Method size
	 * 
	 *  Input space: forall q : Queue, q.arr != null
	 * 
	 *  Partition1 : 0 == q.size == q.arr.length 
	 *  Partition2 : 0 == q.size < q.arr.length
	 *  Partition3 : 0 < q.size < q.arr.length
	 *  Partition4 : 0 < q.size == q.arr.length 
	 */
	@Test
	public void size_test_partition1(){
		int exp_output = 0;
				
		assertEquals(exp_output,q0.size());
	}
	
	@Test
	public void size_test_partition2(){
		int exp_output = 0;
				
		assertEquals(exp_output,q2.size());
	}	
	
	@Test
	public void size_test_partition3(){
		q2.enqueue(7);
		
		int exp_output = 1;
				
		assertEquals(exp_output,q2.size());
	}
	
	@Test
	public void size_test_partition4(){
		q2.enqueue(7);
		q2.enqueue(9);
		
		int exp_output = 2;
				
		assertEquals(exp_output,q2.size());
	}
	
	
	/* Method max
	 * 
	 *  Input space: forall q : Queue, q.arr != null
	 * 
	 *  Partition1 : 0 == q.size == q.arr.length 
	 *  Partition2 : 0 == q.size < q.arr.length
	 *  Partition3 : 0 < q.size < q.arr.length
	 *  Partition4 : 0 < q.size == q.arr.length 
	 */
	@Test
	public void max_test_partition1(){
		int exp_output = 0;
				
		assertEquals(exp_output,q0.max());
	}
	
	@Test
	public void max_test_partition2(){
		int exp_output = 2;
				
		assertEquals(exp_output,q2.max());
	}	
	
	@Test
	public void max_test_partition3(){
		q2.enqueue(7);
		
		int exp_output = 2;
				
		assertEquals(exp_output,q2.max());
	}
	
	@Test
	public void max_test_partition4(){
		q2.enqueue(7);
		q2.enqueue(9);
		
		int exp_output = 2;
				
		assertEquals(exp_output,q2.max());
	}
	
	
	/* Method enqueue
	 * 
	 *  Input space: forall q : Queue, q.arr != null
	 * 
	 *  Partition2 : 0 == q.size < q.arr.length
	 *  Partition3 : 0 < q.size < q.arr.length
	 */	
	@Test
	public void enqueue_test_partition2(){
		q2.enqueue(7);
		
		Queue aux = new Queue(2);
		aux.size = 1;
		aux.arr[0] = 7;
		aux.first = 0;
		aux.last = 1;
		
		assertQueueEquals(q2,aux);
	}
	
	@Test
	public void enqueue_test_partition3(){
		q2.enqueue(7);
		q2.enqueue(9);
		
		Queue aux = new Queue(2);
		aux.size = 2;
		aux.arr[0] = 7;
		aux.arr[1] = 9;
		aux.first = 0;
		aux.last = 0;
		
		assertQueueEquals(aux,q2);
	}
	
	
	/* Method dequeue
	 * 
	 *  Input space: forall q : Queue, q.arr != null
	 * 
	 *  Partition1 : 0 == q.size == q.arr.length 
	 *  Partition2 : 0 == q.size < q.arr.length
	 *  Partition3 : 0 < q.size < q.arr.length
	 *  Partition4 : 0 < q.size == q.arr.length 
	 */	
	@Test
	public void dequeue_test_partition1(){
		Object o = q0.dequeue();
		
		assertNull(o);
	}	
	
	@Test
	public void dequeue_test_partition2(){
		Object o = q2.dequeue();
		
		assertNull(o);
	}
	
	@Test
	public void dequeue_test_partition3(){
		q2.enqueue(7);
		int aux = (int)q2.dequeue();
		
		assertEquals(7,aux);
	}

	@Test
	public void dequeue_test_partition4(){
		q2.enqueue(7);
		q2.enqueue(9);
		
		int aux = (int)q2.dequeue();
		
		assertEquals(7,aux);
	}
	
	//Added to have full statement coverage
	@Test
	public void dequeue_test_extra(){
		q2.enqueue(7);
		q2.enqueue(9);
		
		q2.dequeue();
		int aux = (int)q2.dequeue();
		
		assertEquals(9,aux);
	}
	
	
	//Method added to check equality over Queues
	public void assertQueueEquals(Queue q1,Queue q2){
		assertTrue (q1.first == q2.first 
			        && q1.last == q2.last
			        && q1.size == q2.size);
		assertArrayEquals(q1.arr,q2.arr);
	}
}
